import { Reference, SetOpts, Snapshot } from './reference';
import { VirtualCollection } from './virtual-collection';
import { DocumentReference } from '@google-cloud/firestore';
/**
 * References an item in a virtual collection.
 */
export declare class VirtualReference<T extends object> extends Reference<T> {
    readonly id: string;
    readonly parent: VirtualCollection<T>;
    constructor(id: string, parent: VirtualCollection<T>);
    get path(): string;
    get firestore(): FirebaseFirestore.Firestore;
    delete(opts?: SetOpts): Promise<void>;
    create(data: T, opts?: SetOpts): Promise<T>;
    create(data: Partial<T>, opts?: SetOpts): Promise<Partial<T>>;
    update(data: T, opts?: SetOpts): Promise<T>;
    update(data: Partial<T>, opts?: SetOpts): Promise<Partial<T>>;
    /**
     * Performs a `create()`, `update()`, or `delete()` operation without any coercion or lifecycles.
     * @private
     * @deprecated For internal connector use only
     */
    writeInternal(data: Partial<T> | undefined, editMode: 'create' | 'update'): Promise<void>;
    get(): Promise<Snapshot<T>>;
    isEqual(other: any): boolean;
    /**
     * Allow serialising to JSON.
     */
    toJSON(): string;
    /**
     * Returns a value that can be serialised
     * to Firestore.
     */
    toFirestoreValue(): DocumentReference<T>;
    toString(): string;
}
export declare function makeVirtualSnap<T extends object>(ref: VirtualReference<T>, data: T | undefined): Snapshot<T>;
