"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyManualFilters = void 0;
const _ = require("lodash");
function applyManualFilters(args) {
    let docs = [];
    for (const [id, data] of Object.entries(args.data)) {
        // Must match every 'AND' filter (if any exist)
        // and at least one 'OR' filter (if any exists)
        const snap = {
            id,
            ref: args.model.db.doc(id),
            exists: data != null,
            data: () => data,
        };
        if (args.filters.every(f => f(snap))) {
            docs.push(snap);
        }
    }
    for (const { field, directionStr } of args.orderBy) {
        docs = _.orderBy(docs, d => args.model.getAttributeValue(field, d), directionStr);
    }
    // Offset and limit after sorting
    const offset = Math.max(args.offset || 0, 0);
    const limit = Math.max(args.limit || 0, 0) || docs.length;
    docs = docs.slice(offset, offset + limit);
    return {
        docs,
        empty: docs.length === 0
    };
}
exports.applyManualFilters = applyManualFilters;
